#!/usr/bin/env python
#
# Library for Grove - 3 Axis Accelerometer ADXL372
#

'''
## License

The MIT License (MIT)

Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''


from __future__ import print_function
import time
from grove.i2c import Bus


# Register Address
ADXL372_ADI_DEVID               = 0x00   #  Analog Devices, Inc., accelerometer ID 
ADXL372_MST_DEVID               = 0x01   #  Analog Devices MEMS device ID 
ADXL372_DEVID                   = 0x02   #  Device ID 
ADXL372_REVID                   = 0x03   #  product revision ID
ADXL372_STATUS_1                = 0x04   #  Status register 1 
ADXL372_STATUS_2                = 0x05   #  Status register 2 
ADXL372_FIFO_ENTRIES_2          = 0x06   #  Valid data samples in the FIFO 
ADXL372_FIFO_ENTRIES_1          = 0x07   #  Valid data samples in the FIFO 
ADXL372_X_DATA_H                = 0x08   #  X-axis acceleration data [11:4] 
ADXL372_X_DATA_L                = 0x09   #  X-axis acceleration data [3:0] | dummy LSBs 
ADXL372_Y_DATA_H                = 0x0A   #  Y-axis acceleration data [11:4] 
ADXL372_Y_DATA_L                = 0x0B   #  Y-axis acceleration data [3:0] | dummy LSBs 
ADXL372_Z_DATA_H                = 0x0C   #  Z-axis acceleration data [11:4] 
ADXL372_Z_DATA_L                = 0x0D   #  Z-axis acceleration data [3:0] | dummy LSBs 
ADXL372_X_MAXPEAK_H             = 0x15   #  X-axis MaxPeak acceleration data [15:8] 
ADXL372_X_MAXPEAK_L             = 0x16   #  X-axis MaxPeak acceleration data [7:0] 
ADXL372_Y_MAXPEAK_H             = 0x17   #  X-axis MaxPeak acceleration data [15:8] 
ADXL372_Y_MAXPEAK_L             = 0x18   #  X-axis MaxPeak acceleration data [7:0] 
ADXL372_Z_MAXPEAK_H             = 0x19   #  X-axis MaxPeak acceleration data [15:8] 
ADXL372_Z_MAXPEAK_L             = 0x1A   #  X-axis MaxPeak acceleration data [7:0] 
ADXL372_OFFSET_X                = 0x20   #  X axis offset 
ADXL372_OFFSET_Y                = 0x21   #  Y axis offset 
ADXL372_OFFSET_Z                = 0x22   #  Z axis offset 
ADXL372_X_THRESH_ACT_H          = 0x23   #  X axis Activity Threshold [15:8] 
ADXL372_X_THRESH_ACT_L          = 0x24   #  X axis Activity Threshold [7:0] 
ADXL372_Y_THRESH_ACT_H          = 0x25   #  Y axis Activity Threshold [15:8] 
ADXL372_Y_THRESH_ACT_L          = 0x26   #  Y axis Activity Threshold [7:0] 
ADXL372_Z_THRESH_ACT_H          = 0x27   #  Z axis Activity Threshold [15:8] 
ADXL372_Z_THRESH_ACT_L          = 0x28   #  Z axis Activity Threshold [7:0] 
ADXL372_TIME_ACT                = 0x29   #  Activity Time 
ADXL372_X_THRESH_INACT_H        = 0x2A   #  X axis Inactivity Threshold [15:8] 
ADXL372_X_THRESH_INACT_L        = 0x2B   #  X axis Inactivity Threshold [7:0] 
ADXL372_Y_THRESH_INACT_H        = 0x2C   #  Y axis Inactivity Threshold [15:8] 
ADXL372_Y_THRESH_INACT_L        = 0x2D   #  Y axis Inactivity Threshold [7:0] 
ADXL372_Z_THRESH_INACT_H        = 0x2E   #  Z axis Inactivity Threshold [15:8] 
ADXL372_Z_THRESH_INACT_L        = 0x2F   #  Z axis Inactivity Threshold [7:0] 
ADXL372_TIME_INACT_H            = 0x30   #  Inactivity Time [15:8] 
ADXL372_TIME_INACT_L            = 0x31   #  Inactivity Time [7:0] 
ADXL372_X_THRESH_ACT2_H         = 0x32   #  X axis Activity2 Threshold [15:8] 
ADXL372_X_THRESH_ACT2_L         = 0x33   #  X axis Activity2 Threshold [7:0] 
ADXL372_Y_THRESH_ACT2_H         = 0x34   #  Y axis Activity2 Threshold [15:8] 
ADXL372_Y_THRESH_ACT2_L         = 0x35   #  Y axis Activity2 Threshold [7:0] 
ADXL372_Z_THRESH_ACT2_H         = 0x36   #  Z axis Activity2 Threshold [15:8] 
ADXL372_Z_THRESH_ACT2_L         = 0x37   #  Z axis Activity2 Threshold [7:0] 
ADXL372_HPF                     = 0x38   #  High Pass Filter 
ADXL372_FIFO_SAMPLES            = 0x39   #  FIFO Samples 
ADXL372_FIFO_CTL                = 0x3A   #  FIFO Control 
ADXL372_INT1_MAP                = 0x3B   #  Interrupt 1 mapping control 
ADXL372_INT2_MAP                = 0x3C   #  Interrupt 2 mapping control 
ADXL372_TIMING                  = 0x3D   #  Timing 
ADXL372_MEASURE                 = 0x3E   #  Measure 
ADXL372_POWER_CTL               = 0x3F   #  Power control 
ADXL372_SELF_TEST               = 0x40   #  Self Test 
ADXL372_SRESET                  = 0x41   #  Reset 
ADXL372_FIFO_DATA               = 0x42   #  FIFO Data 

ADXL372_ADI_DEVID_VAL           = 0xAD   #  Analog Devices, Inc., accelerometer ID 
ADXL372_MST_DEVID_VAL           = 0x1D   #  Analog Devices MEMS device ID 
ADXL372_DEVID_VAL               = 0xFA   #  Device ID 
ADXL372_REVID_VAL               = 0x02   #  product revision ID


MEASURE_AUTOSLEEP_MASK          = 0xBF
MEASURE_BANDWIDTH_MASK          = 0xF8
MEASURE_ACTPROC_MASK            = 0xCF
TIMING_ODR_MASK                 = 0x1F
TIMING_WUR_MASK                 = 0xE3
PWRCTRL_OPMODE_MASK             = 0xFC
PWRCTRL_INSTON_THRESH_MASK      = 0xDF
PWRCTRL_INSTON_THRESH_MASK      = 0xDF
PWRCTRL_FILTER_SETTLE_MASK      = 0xEF

MEASURE_AUTOSLEEP_POS           = 6
MEASURE_ACTPROC_POS             = 4
TIMING_ODR_POS                  = 5
TIMING_WUR_POS                  = 2
INSTAON_THRESH_POS              = 5
FIFO_CRL_SAMP8_POS              = 0
FIFO_CRL_MODE_POS               = 1
FIFO_CRL_FORMAT_POS             = 3
PWRCTRL_FILTER_SETTLE_POS       = 4

DATA_READY          = 1
FIFO_READY          = 2
FIFO_FULL           = 4
FIFO_OVERRUN        = 8

# Acceleremoter configuration 
ACT_VALUE           = 30    # Activity threshold value
INACT_VALUE         = 30    # Inactivity threshold value
ACT_TIMER           = 1     # Activity timer value in multiples of 3.3ms
INACT_TIMER         = 1     # Inactivity timer value in multiples of 26ms
ADXL_INT1_PIN       = 7
ADXL_INT2_PIN       = 5

STANDBY_MODE        = 0
WAKEUP_MODE         = 1
INSTANT_ON_MODE     = 2
MEASUREMENT_MODE    = 3

FIFO_BYPASSED = 0
FIFO_STREAMED = 1
FIFO_TRIGGERED = 2
FIFO_OLDEST_SAVED = 3

FIFO_XYZ = 0
FIFO_X = 1
FIFO_Y = 2
FIFO_XY = 3
FIFO_Z = 4
FIFO_XZ = 5
FIFO_YZ = 6
FIFO_XYZ_PEAK = 7


ADXL372_SAMPLE_RATE = (400, 800, 1600, 3200, 6400)
ADXL372_BANDWIDTH = (200, 400, 800, 1600, 3200)
ADXL372_WAKEUP_TIME = (52, 104, 208, 512, 2048, 4096, 8192, 24576)
ADXL372_OP_MODE = (STANDBY_MODE, WAKEUP_MODE, INSTANT_ON_MODE, MEASUREMENT_MODE)


class ADXL372(object):
    def __init__(self, address=0x53, i2c=None):
        self.address = address
        self.bus = Bus(i2c)

        print('ID: {}'.format(self.id))
        self.reset()

        self._timing_control = 0
        self._power_control = 0
        self._measurement_control = 0
        self._sample_rate = 400
        self._bandwidth = 200
        self._mode = STANDBY_MODE


    def read(self):
        raw = self.bus.read_i2c_block_data(self.address, ADXL372_X_DATA_H, 6)
        return self.xyz(raw)

    def read_fifo(self, size):
        data = []
        while size > 32:
            data.extend(self.bus.read_i2c_block_data(self.address, ADXL372_FIFO_DATA, 32))
            size -= 32

        if size:
            data.extend(self.bus.read_i2c_block_data(self.address, ADXL372_FIFO_DATA, size))

        return data

    def reset(self):
        self.write_register(ADXL372_SRESET, 0x52)

    def timing_control(self, sample_rate=400, wakeup_ms=52):
        try:
            sample_bits = ADXL372_SAMPLE_RATE.index(sample_rate)
            wakeup_bits = ADXL372_WAKEUP_TIME.index(wakeup_ms)
        except ValueError:
            print('Supported sample rates: {}'.format(ADXL372_SAMPLE_RATE))
            print('Supported wakeup time: {}'.format(ADXL372_WAKEUP_TIME))
            raise ValueError('Invalid sample rate or wakeup time')
        self._timing_control = (sample_bits << TIMING_ODR_POS) | (wakeup_bits << TIMING_WUR_POS)
        self._sample_rate = sample_rate

        self.write_register(ADXL372_TIMING, self._timing_control)

    def power_control(self, mode=0, low_pass_filter=0, high_pass_filter=0):
        if mode not in ADXL372_OP_MODE:
            raise ValueError('Invalid operating mode')
        value = mode
        if not low_pass_filter:
            value |= 1 << 3
        if not high_pass_filter:
            value |= 1 << 2

        self._mode = mode
        self._power_control = value
        self.write_register(ADXL372_POWER_CTL, value)

    def measurement_control(self, bandwidth=200, low_noise=0, linkloop=0, autosleep=0):
        try:
            value = ADXL372_BANDWIDTH.index(bandwidth)
        except ValueError:
            print('Supported bandwidth: {}'.format(ADXL372_BANDWIDTH))
            raise ValueError('Invalid bandwidth')
        if low_noise:
            value |= 1 << 3
        if linkloop:
            value |= linkloop << 4
        if autosleep:
            value |= 1 << 6

        self._measurement_control = value
        self.write_register(ADXL372_MEASURE, value)

    def fifo_control(self, mode=FIFO_STREAMED, format=FIFO_XYZ, samples=0x80):
        self.write_register(ADXL372_FIFO_SAMPLES, samples & 0xFF)
        self.write_register(ADXL372_FIFO_CTL, ((samples >> 8) & 0x1) | (mode << 1) | (format << 3))

    @property
    def sample_rate(self):
        return self._sample_rate

    @sample_rate.setter
    def sample_rate(self, value):
        self._sample_rate = value
        try:
            simple_bits = ADXL372_SAMPLE_RATE.index(value)
        except ValueError:
            print('Supported sample rates: {}'.format(ADXL372_SAMPLE_RATE))
            raise ValueError('Invalid sample rate')
        self._timing_control = (self._timing_control & TIMING_ODR_MASK) | (simple_bits << TIMING_ODR_POS)
        self.write_register(ADXL372_TIMING, self._timing_control)

    @property
    def bandwidth(self):
        return self._bandwidth

    @bandwidth.setter
    def bandwidth(self, value):
        if value in ADXL372_BANDWIDTH:
            self._bandwidth = value
            bandwidth_bits = ADXL372_BANDWIDTH.index(value)
            self._measurement_control = (self._measurement_control & MEASURE_BANDWIDTH_MASK) | bandwidth_bits
        else:
            print('Supported bandwidth: {}'.format(ADXL372_BANDWIDTH))
            raise ValueError('Invalid bandwidth')

    @property
    def mode(self):
        return self._mode

    @mode.setter
    def mode(self, value):
        if value in ADXL372_OP_MODE:
            self._mode = value
            self._power_control = (self._power_control & PWRCTRL_OPMODE_MASK) | value
            self.write_register(ADXL372_POWER_CTL, value)
        else:
            raise ValueError('Invalid operating mode')

    @property
    def samples_in_fifo(self):
        data = self.bus.read_word_data(self.address, ADXL372_FIFO_ENTRIES_2)
        return ((data & 0x3) << 8) | ((data >> 8) & 0xFF)

    @property
    def status(self):
        return self.bus.read_word_data(self.address, ADXL372_STATUS_1)

    @property
    def id(self):
        return self.bus.read_i2c_block_data(self.address, ADXL372_ADI_DEVID, 4)

    
    def read_register(self, register):
        return self.bus.read_byte_data(self.address, register)

    def write_register(self, register, value):
        self.bus.write_byte_data(self.address, register, value)

    def update_register(self, register, mask, shift, value):
        data = self.read_register(register)
        data = (data & mask) | ((value << shift) & ~mask)
        self.write_register(register, data)

    def dump_registers(self):
        registers = self.bus.read_i2c_block_data(self.address, 0x39, 0x43 - 0x39)
        for register in registers:
            print(hex(register))

    def xyz(self, raw):
        value = [0] * 3
        for i in range(3):
            value[i] = (raw[2*i] << 4) | (raw[2*i+1] >> 4)
            if value[i] & 0xF00:
                value[i] = -((~value[i] & 0xFFF) + 1)

        return value



Grove3AxisAccelerometerADXL372 = ADXL372
Grove = ADXL372


def measurement_mode():
    acc = ADXL372()

    sample_rate = 400  # Hz, options: 400, 800, 1600, 3200, 6400
    acc.sample_rate = sample_rate
    acc.bandwidth = sample_rate / 2
    acc.mode = MEASUREMENT_MODE

    print('By default, the low pass filter and the high pass filter are enabled')

    print('unit - 100 mG')
    while True:
        if acc.status | DATA_READY:
            x, y, z = acc.read()
            print('{:4}, {:4}, {:4}'.format(x, y, z))
        time.sleep(1.)


def wakeup_mode():
    acc = ADXL372()

    wakeup_ms = 2048    #  52, 104, 208, 512, 2048, 4096, 8192, 24576
    acc.timing_control(wakeup_ms=wakeup_ms)
    acc.power_control(mode=WAKEUP_MODE, low_pass_filter=0, high_pass_filter=1)

    print('unit - 100 mG')
    while True:
        if acc.status | DATA_READY:
            x, y, z = acc.read()
            print('{:4}, {:4}, {:4}'.format(x, y, z))
        time.sleep(wakeup_ms / 1000.)

def main():
    acc = Grove3AxisAccelerometerADXL372()

    sample_rate = 400  # Hz
    acc.timing_control(sample_rate=sample_rate)
    acc.measurement_control(bandwidth=sample_rate/2, low_noise=1)
    acc.fifo_control(mode=FIFO_STREAMED, format=FIFO_XYZ, samples=0x81)
    acc.power_control(mode=MEASUREMENT_MODE, low_pass_filter=1, high_pass_filter=0)

    print('unit - 100 mG')
    while True:
        samples = acc.samples_in_fifo
        if samples > 12:
            # To ensure that data is not overwritten and stored out of order, 
            # at least one sample set must be left in the FIFO after every read 
            data = acc.read_fifo((samples // 6 - 1) * 6)
            for i in range(0, len(data), 6):
                x, y, z = acc.xyz(data[i:i+6])
                print('{:4}, {:4}, {:4}'.format(x, y, z))

        time.sleep(1. * 10 / sample_rate)

if __name__ == '__main__':
    main()


#!/usr/bin/env python
#
# This is the code for Grove - UV Sensor
# (https://www.seeedstudio.com/Grove-UV-Sensor-p-1540.html)
#
# This is the library used with Grove Base Hat for RPi.

'''
## License
The MIT License (MIT)
Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 
Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:
The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.
THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import print_function
from grove.i2c import Bus
import time


# I2C address of the device
VEML6070_DEFAULT_ADDRESS				= 0x38

# VEML6070 Command Set
VEML6070_CMD_ACK_DISABLE				= 0x00 # Acknowledge Disable
VEML6070_CMD_ACK_ENABLE					= 0x20 # Acknowledge Enable
VEML6070_CMD_ACK_THD_102				= 0x00 # Acknowledge threshold 102 Steps
VEML6070_CMD_ACK_THD_145				= 0x10 # Acknowledge threshold 145 Steps
VEML6070_CMD_IT_1_2T					= 0x00 # Integration time = 1/2T
VEML6070_CMD_IT_1T						= 0x04 # Integration time = 1T
VEML6070_CMD_IT_2T						= 0x08 # Integration time = 2T
VEML6070_CMD_IT_4T						= 0x0C # Integration time = 4T
VEML6070_CMD_RESERVED					= 0x02 # Reserved, Set to 1
VEML6070_CMD_SD_DISABLE					= 0x00 # Shut-down Disable
VEML6070_CMD_SD_ENABLE					= 0x01 # Shut-down Enable
VEML6070_CMD_READ_LSB					= 0x38 # Read LSB of the data
VEML6070_CMD_READ_MSB					= 0x39 # Read MSB of the data

class VEML6070():
	def __init__(self, address = VEML6070_DEFAULT_ADDRESS):
		self._addr = address
		self._bus  = Bus()
		self.write_command()
	
	def write_command(self):
		"""Select the UV light command from the given provided values"""
		COMMAND_CONFIG = (VEML6070_CMD_ACK_DISABLE | VEML6070_CMD_IT_1_2T | VEML6070_CMD_SD_DISABLE | VEML6070_CMD_RESERVED)
		self._bus.write_byte(VEML6070_DEFAULT_ADDRESS, COMMAND_CONFIG)
	
	def read_uvlight(self):
		"""Read data back VEML6070_CMD_READ_MSB(0x73) and VEML6070_CMD_READ_LSB(0x71), uvlight MSB, uvlight LSB"""
		data0 = self._bus.read_byte(VEML6070_CMD_READ_MSB)
		data1 = self._bus.read_byte(VEML6070_CMD_READ_LSB)
		
		# Convert the data
		uvlight = data0 * 256 + data1
		
		return {'u' : uvlight}


def main():

	veml6070 = VEML6070()

	## Exit handlers ##
	# This function stops python from printing a stacktrace when you hit control-C
	def SIGINTHandler(signum, frame):
		raise SystemExit

	# This function lets you run code on exit, including functions from abpdrrt005pg2a5
	def exitHandler():
		print("Exiting")
		sys.exit(0)
	
	while True:
		light = veml6070.read_uvlight()
		print("UV Value: {0}".format(light['u']))
		print(" *********************************** ")
		time.sleep(1)

if __name__ == '__main__':
	main()


'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''

import io
from setuptools import setup, find_packages


with open('README.md') as f:
    long_description = f.read()

requirements = ['smbus2', 'bmm150', 'bme680', 'sgp30']


def is_pi():
    found = False
    try:
        with open('/proc/cpuinfo', 'r') as cpuinfo:
            for line in cpuinfo:
                if line.startswith('Model'):
                    _, value = line.strip().split(':', 1)
                    value = value.strip()
                    if value.startswith('Raspberry Pi'):
                        found = True
                        break  
    except IOError:
        pass

    return found


if is_pi():
    requirements.append('RPi.GPIO')
    requirements.append('rpi_ws281x')


setup(
    name='Seeed-grove.py',
    version='0.8',
    description='Python library for Seeedstudio Grove devices',
    long_description=long_description,
    long_description_content_type='text/markdown',
    author='Seeed',
    author_email='ruiqian.tang@seeed.cc',
    url='https://github.com/seeed-studio/grove.py',
    packages=find_packages(include=['grove']),
    include_package_data=True,
    install_requires=requirements,
    entry_points={
        'console_scripts': [
            'grove_1wire_thermocouple_amplifier_max31850=grove.grove_1wire_thermocouple_amplifier_max31850:main',
            'grove_3_axis_accelerometer_adxl372=grove.grove_3_axis_accelerometer_adxl372:main',
            'grove_3_axis_compass_bmm150=grove.grove_3_axis_compass_bmm150:main',
            'grove_3_axis_digital_accelerometer=grove.grove_3_axis_digital_accelerometer:main',
            'grove_4_digit_display=grove.grove_4_digit_display:main',
            'grove_6_axis_accel_gyro_bmi088=grove.grove_6_axis_accel_gyro_bmi088:main',
            'grove_12_key_cap_i2c_touch_mpr121=grove.grove_12_key_cap_i2c_touch_mpr121:main',
            'grove_16x2_lcd=grove.display.jhd1802:main',
            'grove_air_quality_sensor_v1_3=grove.grove_air_quality_sensor_v1_3:main',
            'grove_button=grove.grove_button:main',
            'grove_cap_touch_slider_cy8c=grove.grove_cap_touch_slider_cy8c:main',
            'grove_collision_sensor=grove.grove_collision_sensor:main',
            'grove_gesture_sensor=grove.grove_gesture_sensor:main',
            'grove_high_accuracy_temperature=grove.grove_high_accuracy_temperature:main',
            'grove_i2c_color_sensor_v2=grove.grove_i2c_color_sensor_v2:main',
            'grove_i2c_motor_driver=grove.grove_i2c_motor_driver:main',
            'grove_i2c_thermocouple_amplifier_mcp9600=grove.grove_i2c_thermocouple_amplifier_mcp9600:main',
            'grove_imu_9dof_icm20600_ak09918=grove.grove_imu_9dof_icm20600_ak09918:main',
            'grove_lcd_1.2inches=grove.display.sh1107g:main',
            'grove_led=grove.grove_led:main',  
            'grove_light_sensor_v1_2=grove.grove_light_sensor_v1_2:main',
            'grove_loudness_sensor=grove.grove_loudness_sensor:main',
            'grove_mech_keycap=grove.grove_mech_keycap:main',
            'grove_mini_pir_motion_sensor=grove.grove_mini_pir_motion_sensor:main',  
            'grove_moisture_sensor=grove.grove_moisture_sensor:main',     
            'grove_multi_switch=grove.grove_multi_switch:main',
            'grove_multi_switch_poll=grove.button.button_i2c:main',
            'grove_oled_display_128x64=grove.grove_oled_display_128x64:main',                   
            'grove_optical_rotary_encoder=grove.grove_optical_rotary_encoder:main',
            'grove_piezo_vibration_sensor=grove.grove_piezo_vibration_sensor:main',
            'grove_pwm_buzzer=grove.grove_pwm_buzzer:main',
            'grove_recorder_v3_0=grove.grove_recorder_v3_0:main',
            'grove_relay=grove.grove_relay:main',
            'grove_gpio=grove.grove_gpio:main',
            'grove_rotary_angle_sensor=grove.grove_rotary_angle_sensor:main',
            'grove_round_force_sensor=grove.grove_round_force_sensor:main',
            'grove_ryb_led_button=grove.grove_ryb_led_button:main',
            'grove_servo=grove.grove_servo:main',
            'grove_slide_potentiometer=grove.grove_slide_potentiometer:main',
            'grove_sound_sensor=grove.grove_sound_sensor:main',
            'grove_step_counter_bma456=grove.grove_step_counter_bma456:main',
            'grove_switch=grove.grove_switch:main',
            'grove_temperature_humidity_bme680=grove.grove_temperature_humidity_bme680:main',
            'grove_temperature_humidity_sht31=grove.grove_temperature_humidity_sensor_sht3x:main',
            'grove_temperature_sensor=grove.grove_temperature_sensor:main',
            'grove_thumb_joystick=grove.grove_thumb_joystick:main',
            'grove_tilt_switch=grove.grove_tilt_switch:main',
            'grove_time_of_flight_distance=grove.grove_time_of_flight_distance:main',
            'grove_touch_sensor=grove.grove_touch_sensor:main',
            'grove_ultrasonic_ranger=grove.grove_ultrasonic_ranger:main',
            'grove_uv_sensor=grove.grove_uv_sensor:main',
            'grove_water_sensor=grove.grove_water_sensor:main',
            'grove_ws2813_rgb_led_strip=grove.grove_ws2813_rgb_led_strip:main',
            'grove_current_sensor=grove.grove_current_sensor:main'
        ],
    },
    zip_safe=False,
    license = 'MIT',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    keywords=['grove'])



#!/usr/bin/env python
#
# This is the library for Grove Base Hat.
#
# Temper(ature) Classes
#

'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
import math
from grove.adc import ADC

class Temper(object):
    RES_1_2_CELSIUS = 0.5
    RES_1_4_CELSIUS = 0.25
    RES_1_8_CELSIUS = 0.125
    RES_1_16_CELSIUS = 0.0625

    def __init__(self, pin):
        self._resolution = float('nan')

    def fahr2celsius(self, fahr):
        return (fahr - 32.0) / 1.8

    def celsius2fahr(self, celsius):
        return celsius * 1.8 + 32.0

    # arg & return unit in Celsius
    def resolution(self, res = None):
        if res is None:
            return self._resolution
        if self._derive_res(res):
            self._resolution = res
        return self._resolution

    # To be derived
    def _derive_res(self, res):
        return False

    # To be derived
    # return unit in Celsius
    @property
    def temperature(self):
        pass


class TemperTypedNTC(Temper):
    B = 4275.
    R0 = 100000.

    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()
        self._resolution = 1.5  # Celsius
    
    @property
    def temperature(self):
        value = self.adc.read(self.channel)
        if value <= 0 or value >= 1000:
            return float('nan')

        r = 1000. / value - 1.
        r = self.R0 * r

        return 1. / (math.log10(r / self.R0) / self.B + 1 / 298.15) - 273.15




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd. 
#
'''
'''
from __future__ import print_function
from grove.i2c import Bus
from grove.button import Button
import threading
import time

__all__ = ["ButtonTypedI2c", "NAME_5_WAY_SWITCH", "NAME_6_POS_DIP_SWITCH"]

NAME_5_WAY_SWITCH     = "Grove-5-Way-Switch"

""" The Button name to compare with return value of :class:`ButtonTypedI2c.name` """

NAME_6_POS_DIP_SWITCH = "Grove-6-Pos-DIP-Switch"

""" The Button name to compare with return value of :class:`ButtonTypedI2c.name` """

_grove_5way_tactile_keys    = ("KEY A","KEY B","KEY C","KEY D","KEY E")
_grove_6pos_dip_switch_keys = ("POS 1","POS 2","POS 3","POS 4","POS 5","POS 6")

_CMD_GET_DEV_ID     = 0x00
_CMD_GET_DEV_EVENT  = 0x01
_CMD_EVENT_DET_MODE = 0x02
_CMD_BLOCK_DET_MODE = 0x03
_CMD_TEST_GET_VER   = 0xE2

VID_MULTI_SWITCH         = 0x2886
PID_5_WAY_TACTILE_SWITCH = 0x0002
PID_6_POS_DIP_SWITCH     = 0x0003

_CYCLE_PERIOD    = 0.08   # 80 ms

class ButtonTypedI2c(Button):
    '''
    I2C Button/Switch Array Class

    provide event checking ability to derived class,
    should not use directly by end-user.
    The checking events include:

      - Button.EV_SINGLE_CLICK
      - Button.EV_DOUBLE_CLICK
      - Button.EV_LONG_PRESS
      - Button.EV_LEVEL_CHANGED

    Args:
        address(int): optional, the I2C address of the connected device.
        evt_en(bool): optional, default True

            True:  provide event checking ability.

            False: used in poll environment, manually call :class:`ButtonTypedI2c.read`.
    '''
    def __init__(self, address = 0x03, evt_en = True):
        super(ButtonTypedI2c, self).__init__(0)

        self.bus = Bus()
        self._addr = address

        # Initialise the I2C button device
        self.dev_id = 0
        self._probe_uid()
        self._version = 0
        self.version()
        self._size = self.size()

        self._set_mode(True)

        self.__last_evt = None
        self.__last_evt = self.read()

        self.key_names = _grove_5way_tactile_keys
        if self._size == 6:
            self.key_names = _grove_6pos_dip_switch_keys

        self.__thrd_exit = False
        self.__thrd = None
        if not evt_en:
            return

        if self.__thrd is None or not self.__thrd.is_alive():
            self.__thrd = threading.Thread( \
                    target = ButtonTypedI2c.__thrd_chk_evt, \
                    args = (self,))
            self.__thrd.setDaemon(True)
            self.__thrd.start()

    def __del__(self):
        if not self.__thrd:
            return

        self.__thrd_exit = True
        while self.__thrd.isAlive():
            time.sleep(_CYCLE_PERIOD / _CYCLE_UNIT)
        self.__thrd.join()

    # Thread to check events
    def __thrd_chk_evt(self):
        self.__last_time = time.time();
        while not self.__thrd_exit:
        # or self.__state != self.KEY_STATE_IDLE:
            t = time.time()
            dt, self.__last_time = t - self.__last_time, t

            evt = self.read()
            if not evt[0]:
                time.sleep(_CYCLE_PERIOD)
                continue

            for i in range(0, self.size()):
                if evt[i + 1] & ~self.EV_RAW_STATUS:
                    pressed = bool(evt[i + 1] & self.EV_RAW_STATUS)
                    self._index = i
                    self._send_event(evt[i + 1], pressed, t)
            time.sleep(_CYCLE_PERIOD)

    def _probe_uid(self):
        ID_LEN = 4
        for tr in range(4):
            v = self.bus.read_i2c_block_data(self._addr, _CMD_GET_DEV_ID, ID_LEN)
            # print("GET_DEV_ID = {}".format(v))
            did = 0
            for i in range(ID_LEN):
                did = (did >> 8) | (int(v[i]) << 24)
            # print("DEV_ID = {:8X}".format(did))
            if (did >> 16) == VID_MULTI_SWITCH:
                self.dev_id = did
                return self.dev_id
            self.bus.read_byte(self._addr, True)

    def version(self):
        '''
        Get the device firmware version.

        Returns:
            (int): firmware version, the first version is 1
        '''
        VER_LEN = 10
        if not self.dev_id:
            return 0
        v = self.bus.read_i2c_block_data(self._addr, _CMD_TEST_GET_VER, VER_LEN)
        # print("GET_VER = {}".format(str(v)))
        version = v[6] - ord('0')
        version = version * 10 + (v[8] - ord('0'))
        # print("version = {}".format(version))
        self._version = version
        return self._version

    def size(self):
        '''
        Get the button count the device have.

        Returns:
            (int): button count
        '''
        if (self.dev_id >> 16) != VID_MULTI_SWITCH:
            return 0
        if (self.dev_id & 0xFFFF) == PID_5_WAY_TACTILE_SWITCH:
            return 5
        if (self.dev_id & 0xFFFF) == PID_6_POS_DIP_SWITCH:
            return 6
        return 0

    def name(self, index = None):
        '''
        Get the device name or specified button name

        Args:
            index(int): optional, the index number of button to get name.
                        if not specified, return the device name.

        Returns:
            (string): the name of the device or pecified button
        '''
        if (self.dev_id >> 16) != VID_MULTI_SWITCH:
            return "Invalid dev"
        if not index is None:
            if index < 0 or index >= self._size:
                return "Invalid index"
            return self.key_names[index]

        if (self.dev_id & 0xFFFF) == PID_5_WAY_TACTILE_SWITCH:
            return NAME_5_WAY_SWITCH
        if (self.dev_id & 0xFFFF) == PID_6_POS_DIP_SWITCH:
            return NAME_6_POS_DIP_SWITCH
        return "Invalid dev"

    def _set_mode(self, enable):
        if not self.dev_id:
            return None
        v = _CMD_BLOCK_DET_MODE
        if enable:
            v = _CMD_EVENT_DET_MODE
        self.bus.write_byte(self._addr, v)
        return True

    def read(self):
        '''
        Get the button array status

        Returns:
            (list): a list has the size button count + 1
                    item [0] indicate if there is a event (bit 0x80).
                        bit 0x80 set if one or more the switches have event.
                        bit 0x80 clear if no one has event.
                    item [ 1 + `index` ] indicate the event of button specified
                        by index, be bits combination of

                              -  Button.EV_LEVEL_CHANGED
                              -  Button.EV_SINGLE_CLICK
                              -  Button.EV_DOUBLE_CLICK
                              -  Button.EV_LONG_PRESS

        '''
        EVT_LEN = 4
        if not self.dev_id:
            return None
        size = EVT_LEN + self._size
        v = self.bus.read_i2c_block_data(self._addr, _CMD_GET_DEV_EVENT, size)

        if self._version > 1 or self.__last_evt is None:
            return v[EVT_LEN - 1:]

        # Fix: v0.1 will miss event BTN_EV_LEVEL_CHANGED
        #      if this API called frequently.
        for i in range(self._size):
            if (v[EVT_LEN + i] ^ self.__last_evt[1 + i]) & self.EV_RAW_STATUS:
                v[EVT_LEN + i] |= Button.EV_LEVEL_CHANGED
                v[EVT_LEN - 1] |= 0x80
        self.__last_evt = v[EVT_LEN - 1:]
        return v[EVT_LEN - 1:]

    def is_pressed(self, index = 0):
        '''
        Get the button status if it's being pressed ?

        :class:`ButtonTypedI2c.read` must be called before this api call
        when used with poll method object (created with evt_en = False).

        Args:
            index(int): optional, the index number of button to be checked.
                        must be specified for this device.

        Returns:
            (bool):
                True if the button is being pressed.
                False if not.
        '''
        return not bool(self.__last_evt[index + 1] & self.EV_RAW_STATUS)


def main():
    switch = ButtonTypedI2c(evt_en = False)

    print("{} v{} Inserted".format(switch.name(), switch.version()))
    print("A {} Button/Switch Device Ready".format(switch.size()))

    while True:
        evt = switch.read()
        print("EVENT = {}".format(evt))
        for i in range(0, switch.size()):
            print("{}: ".format(switch.name(i)), end='')
            print("{} ".format(switch.is_pressed(i) and "LOW " or "HIGH"), end='')
            if switch.name() == NAME_5_WAY_SWITCH:
                print("{} ".format(evt[i + 1] & switch.EV_RAW_STATUS and "RELEASED" or "PRESSED "), end='')
            elif switch.name() == NAME_6_POS_DIP_SWITCH:
                print("{} ".format(evt[i + 1] & switch.EV_RAW_STATUS and "OFF" or "ON "), end='')
            print(" ", end='')
            print("S" if evt[i + 1] & Button.EV_SINGLE_CLICK  else " ", end='')
            print("D" if evt[i + 1] & Button.EV_DOUBLE_CLICK  else " ", end='')
            print("L" if evt[i + 1] & Button.EV_LONG_PRESS    else " ", end='')
            print("C" if evt[i + 1] & Button.EV_LEVEL_CHANGED else " ", end='')
            print(" ", end='')
        print()
        time.sleep(1.0)

if __name__ == "__main__":
    main()



#!/usr/bin/env python
#
# This is the code for Grove - Step Counter(BMA456).
# (https://www.seeedstudio.com/Grove-Step-Counter-BMA45-p-3189.html)
# which is based on BMA456, an exetremely small, triaxial, 
# low-g high performance accelerations module.
#
# @author Peter Yang <turmary@126.com>
#
# Grove.py is the library for Grove Base Hat which used to
# connect grove sensors for raspberry pi.
#
'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import division
from __future__ import print_function
from ctypes import *
from ctypes import util
import sys
from grove.i2c import Bus

BMA456_I2C_ADDR = 0x19

PERF_MODE_CIC_AVG    = 0
PERF_MODE_CONTINUOUS = 1

BW_OSR4_AVG1   = 0
BW_OSR2_AVG2   = 1
BW_NORMAL_AVG4 = 2
BW_CIC_AVG8    = 3
BW_RES_AVG16   = 4
BW_RES_AVG32   = 5
BW_RES_AVG64   = 6
BW_RES_AVG128  = 7

ODR_0_78_HZ,\
ODR_1_5_HZ, \
ODR_3_1_HZ, \
ODR_6_25_HZ,\
ODR_12_5_HZ,\
ODR_25_HZ,  \
ODR_50_HZ,  \
ODR_100_HZ, \
ODR_200_HZ, \
ODR_400_HZ, \
ODR_800_HZ, \
ODR_1600_HZ = 1,2,3,4,5,6,7,8,9,10,11,12


RANGE_2G, \
RANGE_4G, \
RANGE_8G, \
RANGE_16G = 0,1,2,3

PHONE_CONFIG = 0
WRIST_CONFIG = 1

__c_module = "bma456"

try:
    _ = util.find_library(__c_module)
    # print("libbma = {}".format(_))
    _bma = cdll.LoadLibrary(_)
except Exception:
    print("Error: module lib{}.so unusable, please install lib{}".
          format(__c_module, __c_module))
    sys.exit(1)

class GroveStepCounterBMA456(object):
    def __init__(self, addr = BMA456_I2C_ADDR):
        self._dev = _bma.rpi_bma456_alloc()
        dev_path = "/dev/i2c-{}".format(Bus().bus)
        _bma.rpi_bma456_init(self._dev,
                             dev_path,
                             addr,
                             RANGE_4G,
                             ODR_1600_HZ,
                             BW_NORMAL_AVG4,
                             PERF_MODE_CONTINUOUS)
        _bma.rpi_bma456_enable(self._dev,
                               WRIST_CONFIG,
                               0,
                               1)

    def __del__(self):
        _bma.rpi_bma456_free(self._dev)

    def get_counter(self):
        return _bma.rpi_bma456_get_counter(self._dev)

    def get_temperature(self):
        return _bma.rpi_bma456_get_temperature(self._dev)

    def get_accel(self):
        x, y, z = c_double(), c_double(), c_double()
        _bma.rpi_bma456_get_accel(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

Grove = GroveStepCounterBMA456

def main():
    import time
    print(\
""" Make sure Grove-Step-Counter(BMA456)
   inserted in one I2C slot of Grove-Base-Hat
""")

    snr = GroveStepCounterBMA456()
    while True:
        steps = snr.get_counter()
        print("Steps:  {}".format(steps))
        x, y, z = snr.get_accel()
        print(" AX = %.2fmg AY = %.2fmg AZ = %.2fmg" % (x, y, z))
        time.sleep(1.0)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - `Grove - Slide Potentiometer <https://www.seeedstudio.com/Grove-Slide-Potentiometer-p-1196.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_slide_potentiometer import GroveSlidePotentiometer

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveSlidePotentiometer(PIN)

        while True:
            print('Slide potentiometer value: {}'.format(sensor.value))
            time.sleep(.2)
'''
import math
import time
from grove.adc import ADC

__all__ = ["GroveSlidePotentiometer"]

class GroveSlidePotentiometer(ADC):
    '''
    Grove Slide Poteniometer Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()
    
    @property
    def value(self):
        '''
        Get the position value, max position is 100.0%.

        Returns:
            (int): ratio, 0(0.0%) - 1000(100.0%)
        '''
        return self.adc.read(self.channel)


Grove = GroveSlidePotentiometer


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveSlidePotentiometer(pin)

    while True:
        print('Slide potentiometer value: {}'.format(sensor.value))
        time.sleep(.2)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
#
# This is the code for Grove - IMU 9DOF (ICM20600+AK09918).
# (https://www.seeedstudio.com/Grove-IMU-9DOF-ICM20600-AK0991-p-3157.html)
# which is 9 Degrees of Freedom IMU (Inertial measurement unit) with
# gyroscope, accelerometer and electronic compass, implemented by
# two chips LCM20600 and AK09918.
#
# Author: Peter Yang <turmary@126.com>
#
# Grove.py is the library for Grove Base Hat which used to
# connect grove sensors for raspberry pi.
#
'''
## License

The MIT License (MIT)

Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
Copyright (C) 2018  Seeed Technology Co.,Ltd. 

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
'''
from __future__ import division
from __future__ import print_function
from ctypes import *
from ctypes import util
import sys
from grove.i2c import Bus

ICM20600_I2C_ADDR0   = 0x68
ICM20600_I2C_ADDR1   = 0x69

ICM20600_RANGE_250_DPS, \
ICM20600_RANGE_500_DPS, \
ICM20600_RANGE_1K_DPS,  \
ICM20600_RANGE_2K_DPS   = 0, 1, 2, 3

ICM20600_RANGE_2G, \
ICM20600_RANGE_4G, \
ICM20600_RANGE_8G, \
ICM20600_RANGE_16G = 0, 1, 2, 3

ICM20600_GYRO_RATE_8K_BW_3281, \
ICM20600_GYRO_RATE_8K_BW_250,  \
ICM20600_GYRO_RATE_1K_BW_176,  \
ICM20600_GYRO_RATE_1K_BW_92,   \
ICM20600_GYRO_RATE_1K_BW_41,   \
ICM20600_GYRO_RATE_1K_BW_20,   \
ICM20600_GYRO_RATE_1K_BW_10,   \
ICM20600_GYRO_RATE_1K_BW_5     = 0, 1, 2, 3, 4, 5, 6, 7

ICM20600_ACC_RATE_4K_BW_1046,\
ICM20600_ACC_RATE_1K_BW_420, \
ICM20600_ACC_RATE_1K_BW_218, \
ICM20600_ACC_RATE_1K_BW_99,  \
ICM20600_ACC_RATE_1K_BW_44,  \
ICM20600_ACC_RATE_1K_BW_21,  \
ICM20600_ACC_RATE_1K_BW_10,  \
ICM20600_ACC_RATE_1K_BW_5    = 0, 1, 2, 3, 4, 5, 6, 7

ICM20600_ACC_AVERAGE_4, \
ICM20600_ACC_AVERAGE_8, \
ICM20600_ACC_AVERAGE_16,\
ICM20600_ACC_AVERAGE_32 = 0, 1, 2, 3

ICM20600_GYRO_AVERAGE_1,  \
ICM20600_GYRO_AVERAGE_2,  \
ICM20600_GYRO_AVERAGE_4,  \
ICM20600_GYRO_AVERAGE_8,  \
ICM20600_GYRO_AVERAGE_16, \
ICM20600_GYRO_AVERAGE_32, \
ICM20600_GYRO_AVERAGE_64, \
ICM20600_GYRO_AVERAGE_128 = 0, 1, 2, 3, 4, 5, 6, 7

ICM20600_ICM_SLEEP_MODE,     \
ICM20600_ICM_STANDYBY_MODE,  \
ICM20600_ICM_ACC_LOW_POWER,  \
ICM20600_ICM_ACC_LOW_NOISE,  \
ICM20600_ICM_GYRO_LOW_POWER, \
ICM20600_ICM_GYRO_LOW_NOISE, \
ICM20600_ICM_6AXIS_LOW_POWER,\
ICM20600_ICM_6AXIS_LOW_NOISE = 0, 1, 2, 3, 4, 5, 6, 7

__c_module = "akicm"

try:
    _ = util.find_library(__c_module)
    _akicm = cdll.LoadLibrary(_)
except Exception:
    print("Error: module lib{}.so unusable, please install lib{}".
          format(__c_module, __c_module))
    sys.exit(1)

class ICM20600Cfg(Structure):
    _fields_ = [("gyro_range", c_uint16), \
                ("gyro_rate",  c_uint16), \
                ("gyro_aver",  c_uint16), \
                ("acc_range",  c_uint16), \
                ("acc_rate",   c_uint16), \
                ("acc_aver",   c_uint16), \
                ("power",      c_uint16), \
                ("divider",    c_uint16)  ]

class GroveIMU9DOFICM20600(object):
    def __init__(self, addr = ICM20600_I2C_ADDR1):
        self._dev = _akicm.rpi_icm20600_alloc()
        dev_path = "/dev/i2c-{}".format(Bus().bus)
        icm20600_cfg = ICM20600Cfg(ICM20600_RANGE_2K_DPS,
                                ICM20600_GYRO_RATE_1K_BW_176,
                                ICM20600_GYRO_AVERAGE_1,
                                ICM20600_RANGE_16G,
                                ICM20600_ACC_RATE_1K_BW_420,
                                ICM20600_ACC_AVERAGE_4,
                                ICM20600_ICM_6AXIS_LOW_POWER,
				0)

        _akicm.rpi_icm20600_init(self._dev,
                             dev_path,
                             addr,
                             byref(icm20600_cfg))

    def __del__(self):
        _akicm.rpi_icm20600_free(self._dev)

    def get_temperature(self):
        t = c_double()
        _akicm.rpi_icm20600_get_temperature(self._dev, byref(t))
        return t.value

    def get_accel(self):
        x, y, z = c_double(), c_double(), c_double()
        _akicm.rpi_icm20600_get_accel(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

    def get_gyro(self):
        x, y, z = c_double(), c_double(), c_double()
        _akicm.rpi_icm20600_get_gyro(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

    temperature = get_temperature



AK09918_I2C_ADDR	 = 0x0C

AK09918_POWER_DOWN       = 0x00
AK09918_NORMAL           = 0x01
AK09918_CONTINUOUS_10HZ  = 0x02
AK09918_CONTINUOUS_20HZ  = 0x04
AK09918_CONTINUOUS_50HZ  = 0x06
AK09918_CONTINUOUS_100HZ = 0x08
AK09918_SELF_TEST        = 0x10

AK09918_ERR_OK               = 0 # OK
AK09918_ERR_DOR              = 1 # data skipped
AK09918_ERR_NOT_RDY          = 2 # not ready
AK09918_ERR_TIMEOUT          = 3 # read/write timeout
AK09918_ERR_SELFTEST_FAILED  = 4 # self test failed
AK09918_ERR_OVERFLOW         = 5 # sensor overflow, means |x|+|y|+|z| >= 4912uT
AK09918_ERR_WRITE_FAILED     = 6 # fail to write
AK09918_ERR_READ_FAILED      = 7 # fail to read

class GroveIMU9DOFAK09918(object):
    def __init__(self, addr = AK09918_I2C_ADDR):
        self._dev = _akicm.rpi_ak09918_alloc()
        dev_path = "/dev/i2c-{}".format(Bus().bus)

        _akicm.rpi_ak09918_init(self._dev,
                             dev_path,
                             addr,
                             AK09918_NORMAL)

    def __del__(self):
        _akicm.rpi_ak09918_free(self._dev)

    def mode(self, mode = None):
        if not mode is None:
            _akicm.rpi_ak09918_set_mode(self._dev, mode)
        return _akicm.rpi_ak09918_get_mode(self._dev)

    def reset(self):
        return _akicm.rpi_ak09918_reset(self._dev)

    def is_ready(self):
        if _akicm.rpi_ak09918_is_ready(self._dev) == AK09918_ERR_OK:
            return True
        return False

    def is_skip(self):
        r = _akicm.rpi_ak09918_is_skip(self._dev)
        return (r == AK09918_ERR_DOR)

    def get_magnet(self):
        x, y, z = c_double(), c_double(), c_double()
        _akicm.rpi_ak09918_read(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

    def get_magnet_raw(self):
        x, y, z = c_double(), c_double(), c_double()
        _akicm.rpi_ak09918_read_raw(self._dev,
                                  byref(x), byref(y), byref(z))
        return x.value, y.value, z.value

    def err_string(self, errval):
        return _akicm.rpi_ak09918_err_string(errval)

Grove = GroveIMU9DOFICM20600

def main():
    import time
    print(\
""" Make sure Grove-IMU-9DOF-ICM20600-AK09918
   inserted in one I2C slot of Grove-Base-Hat
""")

    icm = GroveIMU9DOFICM20600()

    ak  = GroveIMU9DOFAK09918()
    ak.mode(AK09918_CONTINUOUS_100HZ)

    while True:
        print("Temperature: {:.2f} C".format(icm.get_temperature()))
        x, y, z = icm.get_accel()
        print(" AX = %7.2f mg  AY = %7.2f mg  AZ = %7.2f mg" % (x, y, z))
        x, y, z = icm.get_gyro()
        print(" GX = %7.2f dps GY = %7.2f dps GZ = %7.2f dps" % (x, y, z))
        if ak.is_ready():
            # if ak.is_skip():
            #     print("*** call get_magnet() too slowly, data droped by AK09918")
            x, y, z = ak.get_magnet()
            print(" MX = %7.2f uT  MY = %7.2f uT  MZ = %7.2f uT" % (x, y, z))        
        time.sleep(1.0)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
This is the code for
    - Grove - Moisture Sensor <https://www.seeedstudio.com/Grove-Moisture-Sensor-p-955.html>`_

Examples:

    .. code-block:: python

        import time
        from grove.grove_moisture_sensor import GroveMoistureSensor

        # connect to alalog pin 2(slot A2)
        PIN = 2

        sensor = GroveMoistureSensor(PIN)

        print('Detecting moisture...')
        while True:
            m = sensor.moisture
            if 0 <= m and m < 300:
                result = 'Dry'
            elif 300 <= m and m < 600:
                result = 'Moist'
            else:
                result = 'Wet'
            print('Moisture value: {0}, {1}'.format(m, result))
            time.sleep(1)
'''
import math
import sys
import time
from grove.adc import ADC

__all__ = ["GroveMoistureSensor"]

class GroveMoistureSensor:
    '''
    Grove Moisture Sensor class

    Args:
        pin(int): number of analog pin/channel the sensor connected.
    '''
    def __init__(self, channel):
        self.channel = channel
        self.adc = ADC()

    @property
    def moisture(self):
        '''
        Get the moisture strength value/voltage

        Returns:
            (int): voltage, in mV
        '''
        value = self.adc.read_voltage(self.channel)
        return value

Grove = GroveMoistureSensor


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.ADC)
    pin = sh.argv2pin()

    sensor = GroveMoistureSensor(pin)

    print('Detecting moisture...')
    while True:
        m = sensor.moisture
        if 0 <= m and m < 300:
            result = 'Dry'
        elif 300 <= m and m < 600:
            result = 'Moist'
        else:
            result = 'Wet'
        print('Moisture value: {0}, {1}'.format(m, result))
        time.sleep(1)

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# This is the library for Grove Base Hat
# which used to connect grove sensors for Raspberry Pi.
'''
This is the code for
    - `Grove - I2C High Accuracy Temperature Sensor(MCP9808) <https://www.seeedstudio.com/Grove-I2C-High-Accuracy-Temperature-Sensor-MCP980-p-3108.html>`_

Examples:
    .. code-block:: python
    import sys
    import time
    from grove.factory import Factory
    from grove.temperature import Temper

    print("Insert Grove - I2C-High-Accuracy-Temperature")
    print("  to Grove-Base-Hat any I2C slot")

    sensor = Factory.getTemper("MCP9808-I2C")
    sensor.resolution(Temper.RES_1_16_CELSIUS)

    print('Detecting temperature...')
    while True:
        print('{} Celsius'.format(sensor.temperature))
        time.sleep(1)
'''
import sys
import time
from grove.factory import Factory
from grove.temperature import Temper

__all__ = ['Temper']

def main():
    print("Insert Grove - I2C-High-Accuracy-Temperature")
    print("  to Grove-Base-Hat any I2C slot")

    #I2C address of temperature device: 0x18 (default) to 0x1F
    i2c_id=0x18

    sensor = Factory.getTemper("MCP9808-I2C", address=i2c_id)
    sensor.resolution(Temper.RES_1_16_CELSIUS)

    print('Detecting temperature...')
    while True:
        print('{} Celsius'.format(sensor.temperature))
        time.sleep(1)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
# Port to support Grove - WS2813 RGB LED Strip Waterproff - XXX LED/m
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
# NeoPixel library strandtest example
# Author: Tony DiCola (tony@tonydicola.com)
#
# Direct port of the Arduino NeoPixel library strandtest example.  Showcases
# various animations on a strip of NeoPixels.
'''
This is the code for
    - `Grove - WS2813 RGB LED Strip Waterproof - 30 LED/m -1m   <https://www.seeedstudio.com/Grove-WS2813-RGB-LED-Strip-Waterproof-30-LED-m-1m-p-3124.html>`_
    - `Grove - WS2813 RGB LED Strip Waterproof - 60 LED/m - 1m  <https://www.seeedstudio.com/Grove-WS2813-RGB-LED-Strip-Waterproof-60-LED-m-1m-p-3126.html>`_
    - `Grove - WS2813 RGB LED Strip Waterproof - 144 LED/m - 1m <https://www.seeedstudio.com/Grove-WS2813-RGB-LED-Strip-Waterproof-144-LED-m-1m-p-3127.html>`_

Examples:

    .. code-block:: python

        import time
        from rpi_ws281x import Color
        from grove.grove_ws2813_rgb_led_strip import GroveWS2813RgbStrip

        # connect to pin 12(slot PWM)
        PIN   = 12
        # For Grove - WS2813 RGB LED Strip Waterproof - 30 LED/m
        # there is 30 RGB LEDs.
        COUNT = 30
        strip = GroveWS2813RgbStrip(PIN, COUNT)

        # Define functions which animate LEDs in various ways.
        def colorWipe(strip, color, wait_ms=50):
            """Wipe color across display a pixel at a time."""
            for i in range(strip.numPixels()):
                strip.setPixelColor(i, color)
                strip.show()
                time.sleep(wait_ms/1000.0)

        print ('Color wipe animations.')
        colorWipe(strip, Color(255, 0, 0))  # Red wipe
        colorWipe(strip, Color(0, 255, 0))  # Blue wipe
        colorWipe(strip, Color(0, 0, 255))  # Green wipe
'''

__all__ = ['GroveWS2813RgbStrip', 'PixelStrip', 'Color']

import time
from rpi_ws281x import PixelStrip, Color

# LED strip configuration
LED_FREQ_HZ    = 800000  # LED signal frequency in hertz (usually 800khz)
LED_DMA        = 10      # DMA channel to use for generating signal (try 10)
LED_BRIGHTNESS = 255     # Set to 0 for darkest and 255 for brightest
LED_INVERT     = False   # True to invert the signal (when using NPN transistor level shift)

class GroveWS2813RgbStrip(PixelStrip):
    '''
    Wrapper Class for Grove - WS2813 RGB LED Strip Waterproof - XXX LED/m

    Args:
        pin(int)  : 12, 18 for RPi
        count(int): strip LEDs count
        brightness(int): optional, set to 0 for darkest and 255 for brightest, default 255
    '''
    def __init__(self, pin, count, brightness = None):
        ws2812_pins = { 12:0, 13:1, 18:0, 19:1}
        if not pin in ws2812_pins.keys():
            print("OneLedTypedWs2812: pin {} could not used with WS2812".format(pin))
            return
        channel = ws2812_pins.get(pin)

        if brightness is None:
            brightness = LED_BRIGHTNESS

        # Create PixelStrip object with appropriate configuration.
        super(GroveWS2813RgbStrip, self).__init__(
            count,
            pin,
            LED_FREQ_HZ,
            LED_DMA,
            LED_INVERT,
            brightness,
            channel
        )

        # Intialize the library (must be called once before other functions).
        self.begin()


# Define functions which animate LEDs in various ways.
def colorWipe(strip, color, wait_ms=50):
    """Wipe color across display a pixel at a time."""
    for i in range(strip.numPixels()):
        strip.setPixelColor(i, color)
        strip.show()
        time.sleep(wait_ms/1000.0)

def theaterChase(strip, color, wait_ms=50, iterations=10):
    """Movie theater light style chaser animation."""
    for j in range(iterations):
        for q in range(3):
            for i in range(0, strip.numPixels(), 3):
                strip.setPixelColor(i+q, color)
            strip.show()
            time.sleep(wait_ms/1000.0)
            for i in range(0, strip.numPixels(), 3):
                strip.setPixelColor(i+q, 0)

def wheel(pos):
    """Generate rainbow colors across 0-255 positions."""
    if pos < 85:
        return Color(pos * 3, 255 - pos * 3, 0)
    elif pos < 170:
        pos -= 85
        return Color(255 - pos * 3, 0, pos * 3)
    else:
        pos -= 170
        return Color(0, pos * 3, 255 - pos * 3)

def rainbow(strip, wait_ms=20, iterations=1):
    """Draw rainbow that fades across all pixels at once."""
    for j in range(256*iterations):
        for i in range(strip.numPixels()):
            strip.setPixelColor(i, wheel((i+j) & 255))
        strip.show()
        time.sleep(wait_ms/1000.0)

def rainbowCycle(strip, wait_ms=20, iterations=5):
    """Draw rainbow that uniformly distributes itself across all pixels."""
    for j in range(256*iterations):
        for i in range(strip.numPixels()):
            strip.setPixelColor(i, wheel((int(i * 256 / strip.numPixels()) + j) & 255))
        strip.show()
        time.sleep(wait_ms/1000.0)

def theaterChaseRainbow(strip, wait_ms=50):
    """Rainbow movie theater light style chaser animation."""
    for j in range(256):
        for q in range(3):
            for i in range(0, strip.numPixels(), 3):
                strip.setPixelColor(i+q, wheel((i+j) % 255))
            strip.show()
            time.sleep(wait_ms/1000.0)
            for i in range(0, strip.numPixels(), 3):
                strip.setPixelColor(i+q, 0)


def main():
    from grove import helper
    from grove.helper import helper
    helper.root_check()

    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.PWM)
    pin = sh.argv2pin(" [led-count]")

    import sys
    count = 30
    if len(sys.argv) >= 3:
        count = int(sys.argv[2])

    strip = GroveWS2813RgbStrip(pin, count)

    print('Press Ctrl-C to quit.')
    try:
        while True:
            print ('Color wipe animations.')
            colorWipe(strip, Color(255, 0, 0))  # Red wipe
            colorWipe(strip, Color(0, 255, 0))  # Blue wipe
            colorWipe(strip, Color(0, 0, 255))  # Green wipe
            print ('Theater chase animations.')
            theaterChase(strip, Color(127, 127, 127))  # White theater chase
            theaterChase(strip, Color(127,   0,   0))  # Red theater chase
            theaterChase(strip, Color(  0,   0, 127))  # Blue theater chase
            print ('Rainbow animations.')
            rainbow(strip)
            rainbowCycle(strip)
            theaterChaseRainbow(strip)

    except KeyboardInterrupt:
        # clear all leds when exit
        colorWipe(strip, Color(0,0,0), 10)


if __name__ == '__main__':
    main()




#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
#
'''
This is the code for
    - `Grove - Tilt Switch <https://www.seeedstudio.com/Grove-Tilt-Switch-p-771.html>`_

Examples:

    .. code-block:: python

        from grove.grove_tilt_switch import GroveTiltSwitch
        import time

        # connect to pin 5 (slot D5)
        PIN = 5

        swicth = GroveTiltSwitch(pin)

        def on_trigger():
            print('Triggered')
        def on_release():
            print("Released.")

        swicth.on_trigger = on_trigger
        swicth.on_release = on_release

        while True:
            time.sleep(1)
'''
import time
from grove.gpio import GPIO

__all__ = ["GroveTiltSwitch"]

class GroveTiltSwitch(GPIO):
    '''
    Grove Tilt Switch class

    Args:
        pin(int): the number of gpio/slot your grove device connected.
    '''
    def __init__(self, pin):
        super(GroveTiltSwitch, self).__init__(pin, GPIO.IN)
        self._on_trigger = None
        self._on_release = None

    @property
    def on_trigger(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a switch shorting/on.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_trigger = callback

            get

            .. code-block:: python

                callobj = obj.on_trigger
        '''
        return self._on_trigger

    @on_trigger.setter
    def on_trigger(self, callback):
        if not callable(callback):
            return

        if self.on_event is None:
            self.on_event = self._handle_event

        self._on_trigger = callback

    @property
    def on_release(self):
        '''
        Property access with
            callback -- a callable function/object,
                        will be called when there is a switch releasing.
            callback prototype:
                callback()
                Returns: none

        Examples:
            set

            .. code-block:: python

                obj.on_release = callback

            get

            .. code-block:: python

                callobj = obj.on_release
        '''
        return self._on_release

    @on_release.setter
    def on_release(self, callback):
        if not callable(callback):
            return

        if self.on_event is None:
            self.on_event = self._handle_event

        self._on_release = callback

    def _handle_event(self, pin, value):

        if value:
            if callable(self._on_trigger):
                self._on_trigger()
        else:
            if callable(self._on_release):
                self._on_release()

Grove = GroveTiltSwitch


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    swicth = GroveTiltSwitch(pin)

    def on_trigger():
        print('Triggered')
    def on_release():
        print("Released.")

    swicth.on_trigger = on_trigger
    swicth.on_release = on_release

    while True:
        time.sleep(1)


if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
# The MIT License (MIT)
#
# Grove Base Hat for the Raspberry Pi, used to connect grove sensors.
# Copyright (C) 2018  Seeed Technology Co.,Ltd.
'''
'''
import time
from grove.gpio import GPIO

__all__ = ["GroveGpio"]

class GroveGpio(GPIO):
    '''
    Class for Grove - Relay

    Args:
        pin(int): number of digital pin the relay connected.
    '''
    def __init__(self, pin):
        super(GroveGpio, self).__init__(pin, GPIO.OUT)

    def on(self):
        '''
        enable/on the relay
        '''
        self.write(1)

    def off(self):
        '''
        disable/off the relay
        '''
        self.write(0)


Grove = GroveGpio


def main():
    from grove.helper import SlotHelper
    sh = SlotHelper(SlotHelper.GPIO)
    pin = sh.argv2pin()

    io = GroveGpio(pin)

    while True:
        try:
            io.on()
            time.sleep(1)
            io.off()
            time.sleep(1)
        except KeyboardInterrupt:
            io.off()
            print("exit")
            exit(1)            

if __name__ == '__main__':
    main()

